[% WRAPPER page.tt %]
  <h2>Modules</h2>

  [% IF modules.size > 0 %]
    <table class="modules">
      [% FOREACH modules %]
        <tr>
          <th><a href="module-[% name %].html">[% label %]</a></th>
          <td class="[% status %]">[% status %]</td>
        </tr>
      [% END %]
    </table>
    
  [% ELSE %]
    <p>There are no modules defined</p>
  [% END %]
[% END %]
