 /***********************************************
 * First of all are some generic style settings 
 * for the whole page. These are specifically
 * designed to work around some fsckups in 
 * netscapes css implemenetation, while keeping
 * forwards compatability
 ***********************************************/

/* Set out the default styles for the document */
body {
  font-family: Verdana,Helvetica,Arial,sans-serif;
  font-size: 76%;
}

p,ul,ol,dl {
  font-size: 1em;
}

th {
  font-weight: bold;
}

/* Set the attributes which are common to all headings */
h1,h2,h3,h4,h5,h6 {
  font-weight: bold;
}

/* Now simply give each heading its own font size */
h1 { font-size: 2em; }
h2 { font-size: 1.8em; }
h3 { font-size: 1.6em; }
h4 { font-size: 1.4em; }
h5 { font-size: 1.2em; }
h6 { font-size: 1em; }

/* Want to make sure that pre uses a fixed width font */
pre { font-family: Courier, monospace; font-size: 1em }

/* All of these tags are use bold font by default */
strong,b { font-weight: bold; }

/* All of these tags use italic font by default */
em,i,cite,dfn,var { font-style: italic; }

/* All of these tags use fixed width font by default */
samp,code,tt,kbd { font-family: Courier, monospace; }

/**************************************************
 * Now the auto build specific styles
 **************************************************/

/*
# Red 2c406d 
# Dark red 4464ac 4464ac
# Gray 777777 777777
# light gray e6e6e6 - e6e6e6
# 
*/

* {
  padding: 0px;
  margin: 0px;
}

body {
  background-color: #e6e6e6;
  color: #000000;
}

h1,h2,h3,h4,h5,h6 {
  padding: 2px;
  padding-left: 5px;
}

h1 {
  background-color: #2c406d;
  color: #ffffff;
  border-bottom: 1px solid #000000;  
  margin-bottom: 10px;
}

h2 {
  background-color: #2c406d;
  color: #000000;
  color: #ffffff;
  border-bottom: 1px solid #000000;  
  border-top: 1px solid #000000;  
}

#status {
  border: 1px solid black;
  background-color: #ffffff;
  color: #000000;
  vertical-align: bottom;
  margin: 1em;
}

#status td, #status th {
  vertical-align: top;
  border-bottom: 1px solid #cccccc;
}

#status td {
  padding-top: 1px;
}

#status th {
  text-align: right;
}


#menu {
  margin: 5px;
  padding: 3px;
  background-color: #4464ac;
  color: #ffffff;
  border-top: 1px solid #777777;
  border-left: 1px solid #777777;
  border-bottom: 1px solid #ffffff;
  border-right: 1px solid #ffffff;
}

#menu li {
  display: inline;
  padding-left: 2em;
//  padding: 3px;
//  margin: 2px;
}

#menu a {
  text-decoration: none;
  color: #ffffff;
  font-size: 1.4em;
}

#menu a:hover {
  color: #000000;
  background-color: #e6e6e6;
}

table.modules {
  background-color: #ffffff;
  color: #000000;
  border: 1px solid #000000;
  margin: 1em;
  width: 90%;
}

table.modules th {
  width: 100%;
  text-align: left;
}

table.modules td {
  text-align: right;
}

table.modules tr {
  vertical-align: middle;
}

table.modules td.failed {
  background-color: #ff0000;
  color: #000000;
  font-size: 1.2em;
  padding: 2px;
}

#footer {
  margin: 5px;
  padding: 3px;
  background-color: #4464ac;
  color: #ffffff;
  border-top: 1px solid #777777;
  border-left: 1px solid #777777;
  border-bottom: 1px solid #ffffff;
  border-right: 1px solid #ffffff;
}

#footer a {
  color: #ffffff;
}

#summary {
  border: 1px solid black;
  background-color: #ffffff;
  color: #000000;
  vertical-align: bottom;
  margin: 1em;
}

#summary td, #summary th {
  vertical-align: top;
  border-bottom: 1px solid #cccccc;
}

#summary th {
  text-align: right
}

#packages, #isos {
  border: 1px solid black;
  margin: 1em;
}

#packages thead, #isos thead {
  background-color: #777777;
  color: #000000;
}

#packages tbody tr.odd, #isos tbody tr.odd {
  background-color: #ffffff;
  color: #000000;
}

#packages tbody tr.even, #isos tbody tr.event {
  background-color: #e6e6e6;
  color: #000000;
}

.filename, .md5sum {
  font-family: courier, monospace;
}

.failed {
  color: #000000;
  background-color: #ff0000;
}

.skipped {
  color: #000000;
  background-color: #ff00ff;
}

.pending {
  color: #000000;
  background-color: #ffff00;
}

.cache {
  color: #000000;
  background-color: #00ff00;
}

.success {
  color: #000000;
  background-color: #00ff00;
}





/**************************************************/
