    <style>
/***********************************************
 * First of all are some generic style settings 
 * for the whole page. These are specifically
 * designed to work around some fsckups in 
 * netscapes css implemenetation, while keeping
 * forwards compatability
 ***********************************************/

/* Set out the default styles for the document */
body,p,th,td {
  font-family: Verdana,Arial,sans-serif;
  font-size: small;
  font-weight: normal;
  font-style: normal;
}

th {
  font-weight: bold;
}

/* Set the attributes which are common to all headings */
h1,h2,h3,h4,h5,h6 {
  font-weight: bold;
}

/* Now simply give each heading its own font size */
h1 { font-size: x-large; }
h2 { font-size: large; }
h3 { font-size: medium; }
h4 { font-size: small; }
h5 { font-size: x-small; }
h6 { font-size: xx-small; }

/* Want to make sure that pre uses a fixed width font */
pre { font-family: Courier, monospace; }

/* All of these tags are use bold font by default */
strong,b { font-weight: bold; }

/* All of these tags use italic font by default */
em,i,cite,dfn,var { font-style: italic; }

/* All of these tags use fixed width font by default */
samp,code,tt,kbd { font-family: Courier, monospace; }
    </style>
