[% WRAPPER page.tt %]
  <h2>Code repositories</h2>

  [% IF repositories.size > 0 %]
    [% FOREACH repositories %]
      [% IF modules.size > 0 %]
        <h3>[% label %]</h3>

        <table class="modules">
          [% FOREACH modules %]
            <tr>
              <th><a href="module-[% name %].html">[% label %]</a></th>
              <td class="[% status %]">[% status %]</td>
            </tr>
          [% END %]
        </table>
      [% END %]
    [% END %]
  [% ELSE %]
    <p>There are no code repositories defined</p>
  [% END %]
[% END %]
